/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
using namespace std;

#include "FLTKWrapper.h"
#include <octave/ov-fcn-handle.h>
#include <octave/symtab.h>
#include <octave/variables.h>

extern "C"
{
#include<stdio.h>
#include<string.h>
}

/* FLTKStaticWrapper methods and classes */
  FLTKStaticWrapper::FLTKStaticWrapper()
  {
    classname=NULL;
    pointer=(void *)0;
    isstatic=false;
  }
  
  /* Well the constructor!! */
    FLTKStaticWrapper::FLTKStaticWrapper(void *ptr,string class_na,bool staticflag)
  {
    std::cout<<"[Static] Wrapper type "<<class_na<<" initialized"<<std::endl;
    pointer=ptr;
    classname=new string(class_na);
    isstatic=staticflag;
  }


  octave_value_list   FLTKStaticWrapper::subsref(const std::string & str,
		       const std::list<octave_value_list>&  idx,
		       int nargout)
  {    
    octave_value_list ovl=idx.front();
    std::cout<<"[StaticWrapper] subsref operator called with #"<<str<<"# "<<std::endl;

#if 0
    //std::cout<<"Operators on Strike. Programs cannot Run"<<std::endl;
    //return octave_value("Static Wrapper:: Cannot Handle");
#endif

    for(int i=0;i<ovl.length();i++)
      {
	std::cout<<i<<" ";
	ovl(i).print(std::cout,true);
	std::cout<<"\n"<<std::endl;
      }
    std::cout<<"Entering evaluator"<<std::endl;

    if(str[0]=='.')
      {
	octave_value_list args;
	int offset=0;
	//std::cout<<"Ready to Jump String "<<*this->classname<<std::endl;
	std::string func_name (*this->classname +"_" + ovl(0).string_value());
	//std::cout<<"Ready to Jump List"<<std::endl;
	std::list<octave_value_list>::const_iterator pidx = idx.begin();
	//std::cout<<"Ready to Jump Eval"<<std::endl;

	// Static methods donot need a pointer to function.
	if(isstatic)	  
	    offset=0;
	else
	  {
	    // Member-methods *DO* need a pointer to function.
	    args(0)=(long int)this->pointer;
	    offset=1;
	  }
	
	if(pidx!=NULL)
	  {
	    pidx++;
	    for(int i=0;i<(*pidx).length();i++)
	      {
		args(i+offset)=(*pidx)(i);
	      }
	  }
	  
	std::cout<<"Evaluating: "<<func_name<<std::endl;

	return feval(func_name,args,nargout);
      }
    return octave_value_list();
  }



    FLTKStaticWrapper::~FLTKStaticWrapper(void)
    {
      /*std::cout<<"Your Object handle is invalidated"<<std::endl;*/
      if( classname != NULL)
	  delete classname;
    }  

DEFINE_OCTAVE_ALLOCATOR (FLTKStaticWrapper);
DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (FLTKStaticWrapper,"FLTKStaticWrapper","FLTKStaticWrapper");



