/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
using namespace std;

#include "FLTKWrapper.h"
#include <octave/ov-fcn-handle.h>
#include <octave/symtab.h>
#include <octave/variables.h>

extern "C"
{
#include<stdio.h>
#include<string.h>
}


static string FLTK_Classes[][6]={
  {"Fl_End","",},
  {"Fl_File_Icon","",},
  {"Fl_Image","",},
  {"Fl_Bitmap","Fl_Image","",},
  {"Fl_XBM_Image","Fl_Bitmap","Fl_Image","",},
  {"Fl_Pixmap","Fl_Image","",},
  {"Fl_GIF_Image","Fl_Pixmap","Fl_Image","",},
  {"Fl_XPM_Image","Fl_Pixmap","Fl_Image","",},
  {"Fl_RGB_Image","Fl_Image","",},
  {"Fl_BMP_Image","Fl_RGB_Image","Fl_Image","",},
  {"Fl_JPEG_Image","Fl_RGB_Image","Fl_Image","",},
  {"Fl_PNG_Image","Fl_RGB_Image","Fl_Image","",},
  {"Fl_PNM_Image","Fl_RGB_Image","Fl_Image","",},
  {"Fl_Shared_Image","Fl_Image","",},
  {"Fl_Tiled_Image","Fl_Image","",},
  {"Fl_Menu_Item","",},
  {"Fl_Preferences","",},
  {"Fl_Text_Buffer","",},
  {"Fl_Tooltip","",},
  {"Fl_Widget","",},
  {"Fl_Box","Fl_Widget","",},
  {"Fl_Browser_","Fl_Widget","",},
  {"Fl_Browser","Fl_Browser_","Fl_Widget","",},
  {"Fl_File_Browser","Fl_Browser","Fl_Browser_","Fl_Widget","",},
  {"Fl_Hold_Browser","Fl_Browser","Fl_Browser_","Fl_Widget","",},
  {"Fl_Multi_Browser","Fl_Browser","Fl_Browser_","Fl_Widget","",},
  {"Fl_Select_Browser ","Fl_Browser","Fl_Browser_","Fl_Widget","",},
  {"Fl_Check_Browser ","Fl_Browser_","Fl_Widget","",},
  {"Fl_Button","Fl_Widget","",},
  {"Fl_Check_Button","Fl_Button","Fl_Widget","",},
  {"Fl_Light_Button","Fl_Button","Fl_Widget","",},
  {"Fl_Repeat_Button","Fl_Button","Fl_Widget","",},
  {"Fl_Return_Button","Fl_Button","Fl_Widget","",},
  {"Fl_Round_Button ","Fl_Button","Fl_Widget","",},
  {"Fl_Chart","Fl_Widget","",},
  {"Fl_Clock","Fl_Widget","",},
  {"Fl_Free","Fl_Widget","",},
  {"Fl_Group","Fl_Widget","",},
  {"Fl_Color_Chooser","Fl_Group","Fl_Widget","",},
  {"Fl_File_Chooser","Fl_Group","Fl_Widget","",},
  {"Fl_Help_Dialog","Fl_Group","Fl_Widget","",},
  {"Fl_Help_View","Fl_Group","Fl_Widget","",},
  {"Fl_Pack","Fl_Group","Fl_Widget","",},
  {"Fl_Scroll","Fl_Group","Fl_Widget","",},
  {"Fl_Tabs","Fl_Group","Fl_Widget","",},
  {"Fl_Text_Display","Fl_Group","Fl_Widget","",},
  {"Fl_Text_Editor ","Fl_Text_Display","Fl_Group","Fl_Widget","",},
  {"Fl_Tile","Fl_Group","Fl_Widget","",},
  {"Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Double_Window","Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Gl_Window","Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Menu_Window","Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Overlay_Window","Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Single_Window ","Fl_Window","Fl_Group","Fl_Widget","",},
  {"Fl_Wizard ","Fl_Group","Fl_Widget","",},
  {"Fl_Input_","Fl_Widget","",},
  {"Fl_Input","Fl_Input_","Fl_Widget","",},
  {"Fl_Float_Input","Fl_Input","Fl_Input_","Fl_Widget","",},
  {"Fl_Int_Input","Fl_Input","Fl_Input_","Fl_Widget","",},
  {"Fl_Multiline_Input","Fl_Input","Fl_Input_","Fl_Widget","",},
  {"Fl_Secret_Input ","Fl_Input","Fl_Input_","Fl_Widget","",},
  {"Fl_Output","Fl_Input_","Fl_Widget","",},
  {"Fl_Multiline_Output ","Fl_Output","Fl_Input_","Fl_Widget","",},
  {"Fl_Menu_","Fl_Input_","Fl_Widget","",},
  {"Fl_Choice","Fl_Menu_","Fl_Input_","Fl_Widget","",},
  {"Fl_Menu_Bar","Fl_Menu_","Fl_Input_","Fl_Widget","",},
  {"Fl_Menu_Button ","Fl_Menu_","Fl_Input_","Fl_Widget","",},
  {"Fl_Positioner","Fl_Input_","Fl_Widget","",},
  {"Fl_Progress","Fl_Input_","Fl_Widget","",},
  {"Fl_Timer","Fl_Input_","Fl_Widget","",},
  {"Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Adjuster","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Counter","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Dial","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Roller","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Slider","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Scrollbar","Fl_Slider","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Value_Slider ","Fl_Slider","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Value_Input","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"Fl_Value_Output ","Fl_Valuator","Fl_Input_","Fl_Widget","",},
  {"",}
};

/* 
   This function looksup, a superclass from this type hierarchy,
   for delegating member functions, of parent types.

   For finding the bottom-most/most-recent parent, you may actually
   pass parent="". To find parent type of 'type' above the type-x
   pass parent='type-x'.

   return value='type-y' indicates success, matched.
   return value='' indicates, weve reached top of order, sorry
   still couldnot find parenty-type for, the function.
 */
static const string FLTK_Find_Super_Class(string type,string parent="")
{
  int i=0;
  int index=-1;
  
  for(i=0;i<80;i++)
    {
      if(FLTK_Classes[i][0]==type)
	{
	  index=i;
	  break;
	}
    }
  if(i==80 && index==-1)
    {
      std::cout<<"Cannot find specified type "<<type<<std::endl;
      return string("");
    }

  /* Most recent parent */
  if(parent=="")
    {
      std::cout<<"Most Recent Parent of type ["<<type<<"] is ["<<FLTK_Classes[index][1]<<" "<<std::endl;
      return FLTK_Classes[index][1];
    }
    
  for(int j=1;j<6;j++)
    {
      if (FLTK_Classes[index][j]=="")
	break;
      
      if(FLTK_Classes[index][j]==parent)
	{
	  if( (j+1)<6)
	    return FLTK_Classes[index][j+1];
	  else
	    return "";
	}
      
    }
  
  return "";
}


  FLTKSet::FLTKSet()
  {
  };

  FLTKSet::FLTKSet(void *ptr,string fcn,octave_value ud)
  {
    pointer=ptr;
    fcnname=fcn;
    user_data=ud;
  };

/* FLTKCallback class methods */
std::list<FLTKSet *>  FLTKCallback::pointer_list; /*List of all pointer data */
int FLTKCallback::current_fcn_1=0;
int FLTKCallback::current_fcn_2=0;


  FLTKCallback::FLTKCallback()
    {
      pointer=0;
      ismethod=false;
      userdata=0;
    };
  FLTKCallback::FLTKCallback(void *invokingtype,string fname)
    {
      pointer=invokingtype;
      fcnname=fname;
    };
  FLTKCallback::FLTKCallback(void *invokingtype,string fname,void *user_data)
    {
      pointer=invokingtype;
      fcnname=fname;
      userdata=user_data;
    }
void FLTKCallback::F1Callback_handler(Fl_Widget *w, void *data)
    {
      FLTKSet *set=(FLTKSet *)data;
      octave_value_list args;
    
      args(0)=octave_value(new FLTKWrapper(w,"Fl_Widget"));
      args(1)=set->user_data;
      
      feval(set->fcnname,args);
    }

/* FLTKWrapper class methods */

std::list<FLTKWrapper*> FLTKWrapper::pointer_list;



/* FLTKWrapper class & method */

double  FLTKWrapper::double_value()
{
  long int x=(long int) this->getPointer();
  return (double)x;
}

long  FLTKWrapper::long_value()
{
  return (long )this->getPointer();
}


/*
  Take the instance-class name & its function arguments.
  Try to recursively climbup a [inheritance] tree, and find if a function
  exists.

  If it exists delegate the call to this function, on that parent
  class. Return this string in a  $class_$function format.

  If not found in any of the subclasses, then return '' empty
  string.
 */
static const string FLTKWrapper_Find_Delegate_Type(string classname,
					     string function)
{
  /* See if this method belongs to a top-order/parent  class,
     and suitably, delegate the call */
  string func_name;
  func_name=classname +"_" +function;

  std::cout<<"Checking if Function  "<<func_name<<"Is defined:"<<std::endl;
  /* Rewrite this as a function, and dump output on,
     meeting topmost level [an error], or meeting
     the correct parent function
  */
	    
  if( global_sym_tab->lookup(func_name,false,false)==NULL
      && top_level_sym_tab->lookup(func_name,false,false)==NULL
      && curr_sym_tab->lookup(func_name,false,false)==NULL
      && fbi_sym_tab->lookup(func_name,false,false)==NULL )
    {
      std::cout<<"Function *NOT* defined: "<<func_name<<std::endl;    
      /* Chain Up to Widget:*/      
      string parent="";
	
      /* Recurse */
      bool found=false;

      while(true)
	{
	  parent=FLTK_Find_Super_Class(classname,parent);
	  if (parent=="")
	    {
	      std::cout<<"No Superclass defines this function: Do Not Invoke this function"<<func_name<<std::endl;
	      func_name="";
	      break;
	    }
	  
	  /* classname becomes, super-class/parent name */
	  func_name=FLTKWrapper_Find_Delegate_Type(parent,function);
	  if(func_name!="")
	    {
	      found=true;
	      std::cout<<"A Superclass ["<<parent<<"] defines this function: Can Invoke this function"<<func_name<<std::endl;
	      break;
	    }
	}
      if (found)
	{
	  return func_name;
	}
      else
	{
	  return "";
	}
    }
  
  std::cout<<"Function is defined: "<<func_name<<std::endl;
  return func_name;
}

octave_value_list FLTKWrapper::subsref(const std::string & str,
				       const std::list<octave_value_list>&  idx,
				       int nargout)
  {    
    octave_value_list ovl=idx.front();
    std::cout<<"[FLTKWrapper [type] subsref operator called with #"<<str<<"# "<<std::endl;
    // std::cout<<"Operators on Strike. Programs cannot Run"<<std::endl;
    // return octave_value("Cannot Handle");
    
    for(int i=0;i<ovl.length();i++)
      {
	std::cout<<i<<" ";
	ovl(i).print(std::cout,true);
	std::cout<<"\n"<<std::endl;
      }
    std::cout<<"Entering evaluator"<<std::endl;

    if(str[0]=='.')
      {
	octave_value_list args;
	int offset=0;
	//std::cout<<"Ready to Jump String "<<*this->classname<<std::endl;
	std::string func_name;
	//std::cout<<"Ready to Jump List"<<std::endl;
	std::list<octave_value_list>::const_iterator pidx = idx.begin();
	//std::cout<<"Ready to Jump Eval"<<std::endl;

	bool iscallback=false;

	/* Handle Callbacks */
	if(ovl(0).string_value()=="callback")
	  {
	    func_name="FLTKWrapper_callback";
	    iscallback=true;
	  }
	else	  
	  {
	    func_name=FLTKWrapper_Find_Delegate_Type(*this->classname,ovl(0).string_value());
	  }


	std::cout<<"Evaluating: "<<func_name<<std::endl;
	
	// Static methods donot need a pointer to function.
	if(isstatic && !iscallback)	  
	    offset=0;
	else
	  {
	    // Member-methods *DO* need a pointer to function.
	    // callbacks will also need a pointer to instance, at function.
	    args(0)=(long int)this->pointer;
	    offset=1;
	  }
	
	if(pidx!=NULL)
	  {
	    pidx++;
	    for(int i=0;i<(*pidx).length();i++)
	      {
		args(i+offset)=(*pidx)(i);
	      }
	  }
	       
	std::cout<<"Executing Function: "<<func_name<<std::endl;
	return feval(func_name,args,nargout);
      }

    return octave_value_list();
  }

    void* FLTKWrapper::getPointer(void)
    {
      return this->pointer;
    }

string FLTKWrapper::getClassname(void)
    {
      return *this->classname;
    }

  FLTKWrapper::FLTKWrapper()
  {
    classname=NULL;
    pointer=(void *)0;
    isstatic=false;
  }

  /* Well the constructor */
  FLTKWrapper::FLTKWrapper(void *ptr,string class_na,bool staticflag)
  {
    std::cout<<"[Dynamic] Wrapper type "<<class_na<<" initialized"<<std::endl;
    pointer=ptr;
    classname=new string(class_na);
    isstatic=staticflag;
    FLTKWrapper::pointer_list.push_back(this);
  }

 string FLTKWrapper::GetClassName(void * pointer)
    {
      for(std::list<FLTKWrapper *>::const_iterator stli=
	    FLTKWrapper::pointer_list.begin();
	  stli!=FLTKWrapper::pointer_list.end();
	  stli++)
	{
	  if((unsigned long int)(*stli)->pointer == (unsigned long int)pointer)
	    {
	      return (*(*stli)->classname);
	    }
	}
      
      return string("");
    }

 bool FLTKWrapper::ExistObjectType(void * pointer,string classname)
    {
      //bool searchflag=false;
      for(std::list<FLTKWrapper *>::const_iterator stli=FLTKWrapper::pointer_list.begin();
	  stli!=FLTKWrapper::pointer_list.end();
	  stli++)
	{
	  if(((unsigned long int)(*stli)->pointer == (unsigned long int)pointer)
	     && *((*stli)->classname) == classname)
	    {
	      return true;
	    }
	}
      return false;
    }

 bool FLTKWrapper::Exists(void * pointer)
    {
      //bool searchflag=false;
      for(std::list<FLTKWrapper *>::const_iterator stli=
	    FLTKWrapper::pointer_list.begin();
	  stli!=FLTKWrapper::pointer_list.end();
	  stli++)
	{
	  if( (unsigned long int) (*stli)->pointer == (unsigned long int)pointer)	    
	    {
	      return true;
	    }
	}
      return false;
    }

 FLTKWrapper::~FLTKWrapper(void)
    {
      /*std::cout<<"Your Object handle is invalidated"<<std::endl;*/
      if( classname != NULL)
	  delete classname;

      FLTKWrapper::pointer_list.remove(this);

#if 0
      for(std::list<FLTKWrapper *>::const_iterator stli=FLTKWrapper::pointer_list.begin();
	  stli!=FLTKWrapper::pointer_list.end();
	  stli++)
	{
	  if((unsigned long int)stli->pointer == (unsigned long int)pointer)
	    {
	    }
	}
#endif

    }  



DEFINE_OCTAVE_ALLOCATOR (FLTKWrapper);
DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (FLTKWrapper,"FLTKWrapper","FLTKWrapper");


DEFUN_DLD(FLTKWrapper_Convert,args,,"FLTKWrapper Convert(FLTKWrapper obj,string newtype)")
{
  if(args.length()!=2)
    {
      return octave_value();
    }
  void *pointer=0;

  string classname = args(1).string_value();
  if(args(0).type_name() == "FLTKWrapper")
    {
      FLTKWrapper &flw=(FLTKWrapper &)args(0).get_rep();
      pointer = flw.getPointer();
    }
  else
    pointer = (void *)args(0).int_value();
  return octave_value(new FLTKWrapper(pointer,classname));
}


DEFUN_DLD(FLTKWrapper_callback,args,,"")
{
  /*
    Internal call: Donot attempt from [User Space] Octave.
    args(0) = widget - instance - pointer
    args(1) = octave function name 
    args(2) = extra data to be supplied
  */

#if 0
  std::list<FLTKSet *> *pointer_list=null;
#endif

  if(args.length()!=3)
    {
      std::cout<<"Donot Call from Octave. cannot process"<<std::endl;
      return octave_value(-1);
    }
  
  Fl_Widget *w=(Fl_Widget *)args(0).long_value();  

#if 0
  if(w->user_data()==NULL)
    {
      /* Create a New Vector Table. */
      pointer_list=new std::list<FLTKSet *>;      
    }
  else
    {
      pointer_list=(std::list<FLTKSet *>)w->user_data();
    }
#endif
  
  FLTKSet *s=new FLTKSet((void *)w,args(1).string_value(),args(2));

#if 0
  pointer_list::push_back(s);
#endif

  w->callback((Fl_Callback *)FLTKCallback::F1Callback_handler,(void *)s);
  return octave_value();
}

/* FLTK Types */
DEFUN_DLD(FLTKWrapper,args,,"")
{
  FLTKWrapper *flw=new FLTKWrapper();

  /* Utility: sub-classing & super-classing functions for FLTK */
  octave_dynamic_loader::load("FLTKWrapper_Convert","");
  octave_dynamic_loader::load("FLTKWrapper_callback","");
  return octave_value(new FLTKStaticWrapper(flw,"FLTKWrapper"));
}

