/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}

DEFUN_DLD( Fl_Bitmap_copy , args , , " virtual Fl_Bitmap_copy (); " )
{
Fl_Bitmap *   _Fl_BitmapPtr ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  virtual Fl_Bitmap_copy (); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Bitmap_copy "<<std::endl;
#endif /* DEBUG */
 
 _Fl_BitmapPtr =( Fl_Bitmap * )args( 0 ).long_value();
 Fl_Image *  ret_type= _Fl_BitmapPtr->copy ();
 
 return octave_value(new FLTKWrapper(ret_type,"Fl_Image"));
}


DEFUN_DLD( Fl_Bitmap_draw , args , , " void Fl_Bitmap_draw (int X,int Y); " )
{
Fl_Bitmap *   _Fl_BitmapPtr ;
int   X ;
int   Y ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  void Fl_Bitmap_draw (int X,int Y); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
 std::cout<<" Fl_Bitmap_draw "<<std::endl;
#endif /* DEBUG */
 _Fl_BitmapPtr =( Fl_Bitmap * )args( 0 ).long_value();
 X = (  int  ) args( 1 ).int_value();
 Y = (  int  ) args( 2 ).int_value();
 _Fl_BitmapPtr->draw ( X,Y);
 return octave_value();
}

/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Bitmap , args , , " void Fl_Bitmap(void); " )
{
  /* Fl_Bitmap(Matrix,true/false) */
  if (args.length()>=1 && args.length()<=2)
    {
      Fl_Bitmap *bitmap=NULL;
      Matrix m;
      int r=0,c=0;
      int i=0,j=0;
      bool signedval=false;
          
      if (args.length()==2 && args(1).bool_value() == true)
	{
	  /* Signed Values */
	  signedval=true;
	}
      r=m.rows();
      c=m.cols();
      unsigned char array [r*c];

      m=args(0).matrix_value();
      for(i=0;i<r;i++)
	for(j=0;j<c;j++)
	  {
	    if(signedval)
	      array[j+i*c]=(char)m.elem(i,j);
	    else
	      array[j+i*c]=(unsigned char)m.elem(i,j);
	  }

      /* Choose Unsigned or Signed */
      /* Default, Unsigned */
      if(signedval)
	  bitmap=new Fl_Bitmap(array,c,r);
      else
	  bitmap=new Fl_Bitmap(array,c,r);

      return octave_value(new FLTKWrapper(bitmap,"Fl_Bitmap"));
    }

  octave_dynamic_loader::load("Fl_Bitmap_Fl_Image" ,"");
  octave_dynamic_loader::load("Fl_Bitmap_copy" ,"");
  octave_dynamic_loader::load("Fl_Bitmap_draw" ,"");

/* Virtual Methods */
#if 0 
  octave_dynamic_loader::load("Fl_Bitmap_draw" ,"");
  octave_dynamic_loader::load("Fl_Bitmap_label" ,"");
  octave_dynamic_loader::load("Fl_Bitmap_label" ,"");
  octave_dynamic_loader::load("Fl_Bitmap_uncache" ,"");
#endif

  return octave_value("Code generated onSat Jul 30 23:28:31 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_BitmapLibrary!");
}

/* 
What do we do to virtual types ?? 
Ignore them ATM.
*/
