/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}
DEFUN_DLD( Fl_Counter_handle , args , , " int Fl_Counter_handle (int ); " )
{
Fl_Counter *   _Fl_CounterPtr ;
int   _int0 ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  int Fl_Counter_handle (int ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_handle "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
_int0 = (  int  ) args( 1 ).int_value();
long int ret_type=(long int) _Fl_CounterPtr->handle ( _int0);
return octave_value(ret_type);
}
DEFUN_DLD( Fl_Counter_lstep , args , , " void Fl_Counter_lstep (double a); " )
{
Fl_Counter *   _Fl_CounterPtr ;
double   a ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  void Fl_Counter_lstep (double a); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_lstep "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
a = (  double  ) args( 1 ).double_value();
_Fl_CounterPtr->lstep ( a);
return octave_value();
}

DEFUN_DLD( Fl_Counter_step , args , , " void Fl_Counter_step (double a); " )
{
Fl_Counter *   _Fl_CounterPtr ;
double   a ;
double   b ;
 if(args.length()!= 2 && args.length()!= 3)
  {
   std::cout<< "eg:  void Fl_Counter_step (double a); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_step "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
 if(args.length()==2)
   {
     a = (  double  ) args( 1 ).double_value();
     _Fl_CounterPtr->step ( a); 
     return octave_value();
   } 
 
 a = (  double  ) args( 1 ).double_value();
 b = (  double  ) args( 2 ).double_value();
 _Fl_CounterPtr->step ( a,b);

 return octave_value();
}

DEFUN_DLD( Fl_Counter_textfont , args , , " void Fl_Counter_textfont (uchar s); " )
{
Fl_Counter *   _Fl_CounterPtr ;
uchar   s ;
 if(args.length()!= 2 && args.length()!= 1 )
  {
   std::cout<< "eg:  void Fl_Counter_textfont (uchar s); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_textfont "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
 if(args.length()==2)
   { 
     s = (  uchar  ) args( 1 ).long_value();
     _Fl_CounterPtr->textfont ( s);
     return octave_value();
   }
 long int ret_type=(long int) _Fl_CounterPtr->textfont ( );
 return octave_value(ret_type);
}

DEFUN_DLD( Fl_Counter_textsize , args , , " void Fl_Counter_textsize (uchar s); " )
{
Fl_Counter *   _Fl_CounterPtr ;
uchar   s ;
 if(args.length()!= 2 && args.length()!= 1 )
  {
   std::cout<< "eg:  void Fl_Counter_textsize (uchar s); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_textsize "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
 if(args.length()==2)
   { 
     s = (  uchar  ) args( 1 ).long_value();
     _Fl_CounterPtr->textsize ( s);
     return octave_value();
   }
 long int ret_type=(long int) _Fl_CounterPtr->textsize ( );
 return octave_value(ret_type);
}

DEFUN_DLD( Fl_Counter_textcolor , args , , " void Fl_Counter_textcolor (s ); " )
{
  Fl_Counter *   _Fl_CounterPtr ;
  long int   _s0 ;
 if(args.length()!= 2 ||  args.length()!= 1 )
  {
   std::cout<< "eg:  void Fl_Counter_textcolor (s ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Counter_textcolor "<<std::endl;
#endif /* DEBUG */
_Fl_CounterPtr =( Fl_Counter * )args( 0 ).long_value();
 if( args.length()== 2 )
   {
     _s0 = (  long int  ) args( 1 ).long_value();
     _Fl_CounterPtr->textcolor ( _s0);
     return octave_value();
   }
 long int ret_type=(long int) _Fl_CounterPtr->textcolor ( );
 return octave_value(ret_type);
}
/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Counter , args , , " void Fl_Counter(void); " )
{
  if (args.length() >= 4 && args.length() <= 5)
    {
      char *str=NULL;
      if(args.length()==5)
        str=strdup(args(4).string_value().c_str());
      Fl_Counter *flb=new Fl_Counter(args(0).int_value(),args(1).int_value(),
				     args(2).int_value(),args(3).int_value(),str);
      return octave_value(new FLTKWrapper(flb,"Fl_Counter"));
    }


  octave_dynamic_loader::load("Fl_Counter_handle" ,"");
  octave_dynamic_loader::load("Fl_Counter_lstep" ,"");
  octave_dynamic_loader::load("Fl_Counter_step" ,"");
  octave_dynamic_loader::load("Fl_Counter_textfont" ,"");
  octave_dynamic_loader::load("Fl_Counter_textsize" ,"");
  octave_dynamic_loader::load("Fl_Counter_textcolor" ,"");
  return octave_value("Code generated onSat Jul 30 23:28:31 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_CounterLibrary!");
}
