/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}

DEFUN_DLD(Fl_File_Input_down_box,args,,
	  "Fl_Boxtype	down_box() const { return (Fl_Boxtype)down_box_; }"
	  "void		down_box(Fl_Boxtype b) { down_box_ = b; }"
	  )
{
  Fl_File_Input *   fptr ;
  if(args.length()!=1 || args.length()!=2)
    {      
      std::cout<< "eg:  void Fl_File_Input_down_box(); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_File_Input_down_box "<<std::endl;
#endif /* DEBUG */

  fptr = ( Fl_File_Input * )args( 0 ).long_value();
  if(args.length() == 1)
    {
      Fl_Boxtype box=fptr->down_box();
      return octave_value(box);
    }
  Fl_Boxtype box=(Fl_Boxtype)args(2).long_value();
  fptr->down_box(box);
  return octave_value();
}


DEFUN_DLD(Fl_File_Input_errorcolor,args,, " Fl_Color	errorcolor() const { return errorcolor_; } void	errorcolor(Fl_Color c) { errorcolor_ = c; }"
	  )
{
  Fl_File_Input *   fptr ;
  if(args.length()!=1 || args.length()!=2)
    {      
      std::cout<< "eg:  void Fl_File_Input_errorcolor(); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_File_Input_errorcolor "<<std::endl;
#endif /* DEBUG */

  fptr = ( Fl_File_Input * )args( 0 ).long_value();
  if(args.length() == 1)
    {
      Fl_Color box=fptr->errorcolor();
      return octave_value(box);
    }
  Fl_Color box=(Fl_Color)args(2).long_value();
  fptr->errorcolor(box);
  return octave_value();
}

DEFUN_DLD(Fl_File_Input_value,args,, "int	value(const char*); int	value(const char*, int);  const char	*value() { return Fl_Input_::value();")
{
  Fl_File_Input *   fptr ;
  if(args.length()!=1 || args.length()!=2 || args.length()!=3)
    {      
      std::cout<< "eg:  void Fl_File_Input_value(); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_File_Input_errorcolor "<<std::endl;
#endif /* DEBUG */

  fptr = ( Fl_File_Input * )args( 0 ).long_value();
  if(args.length() == 1)
    {
      const char *val=fptr->value();
      return octave_value(new string(val));
    }
  else if(args.length() == 2)
    {
      int x=fptr->value(args(1).string_value().c_str());
      return octave_value(x);
    }

  int intval=fptr->value(args(1).string_value().c_str(),args(2).int_value());
  return octave_value(intval);  
}

/* Overridden functions */
/* Linked Separately */
DEFUN_DLD(Fl_File_Input , args , , " void Fl_File_Input(void); " )
{
  if (args.length() >= 4 && args.length() <= 5)
    {
      char *str=NULL;
      if(args.length()==5)
	str=strdup(args(4).string_value().c_str());
      
      Fl_File_Input *flb=new Fl_File_Input(args(0).int_value(),
					   args(1).int_value(),
					   args(2).int_value(),
					   args(3).int_value(),str);
      
      return octave_value(new FLTKWrapper(flb,"Fl_File_Input"));
    }

  octave_dynamic_loader::load("Fl_File_Input_down_box","");
  octave_dynamic_loader::load("Fl_File_Input_errorcolor","");
  octave_dynamic_loader::load("Fl_File_Input_value","");
  
  std::cout<<"Code generated onSat Jul 30 23:28:32 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_File_InputLibrary!\n";
  
  return octave_value(new FLTKStaticWrapper(0,"Fl_File_Input"));
}
