/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}

DEFUN_DLD( Fl_Overlay_Window_flush , args , ,
	   " void Fl_Overlay_Window_flush ( ); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  if(args.length()!= 1 )
    {
      std::cout<< "eg:  void Fl_Overlay_Window_flush ( ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Overlay_Window_flush "<<std::endl;
#endif /* DEBUG */
  _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();
  _Fl_Overlay_WindowPtr->flush ( );
  return octave_value();
}

DEFUN_DLD( Fl_Overlay_Window_hide , args , , " void Fl_Overlay_Window_hide ( ); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  if(args.length()!= 1 )
    {
      std::cout<< "eg:  void Fl_Overlay_Window_hide ( ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Overlay_Window_hide "<<std::endl;
#endif /* DEBUG */
  _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();
  _Fl_Overlay_WindowPtr->hide ( );
  return octave_value();
}

DEFUN_DLD( Fl_Overlay_Window_resize , args , , 
	   " void Fl_Overlay_Window_resize (int ,int ,int ,int ); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  int   _int0 ;
  int   _int1 ;
  int   _int2 ;
  int   _int3 ;
  if(args.length()!= 5 )
    {
      std::cout<< "eg:  void Fl_Overlay_Window_resize (int ,int ,int ,int ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
std::cout<<" Fl_Overlay_Window_resize "<<std::endl;
#endif /* DEBUG */
 _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();
 _int0 = (  int  ) args( 1 ).int_value();
 _int1 = (  int  ) args( 2 ).int_value();
 _int2 = (  int  ) args( 3 ).int_value();
 _int3 = (  int  ) args( 4 ).int_value();
 _Fl_Overlay_WindowPtr->resize ( _int0,_int1,_int2,_int3);
 return octave_value();
}

DEFUN_DLD( Fl_Overlay_Window_can_do_overlay , args , , 
	   " int Fl_Overlay_Window_can_do_overlay ( ); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  if(args.length()!= 1 )
    {
      std::cout<< "eg:  int Fl_Overlay_Window_can_do_overlay ( ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Overlay_Window_can_do_overlay "<<std::endl;
#endif /* DEBUG */
  _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();
  long int ret_type=(long int) _Fl_Overlay_WindowPtr->can_do_overlay ( );
  return octave_value(ret_type);
}

DEFUN_DLD( Fl_Overlay_Window_redraw_overlay , args , ,
	   " void Fl_Overlay_Window_redraw_overlay ( ); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  if(args.length()!= 1 )
    {
      std::cout<< "eg:  void Fl_Overlay_Window_redraw_overlay ( ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Overlay_Window_redraw_overlay "<<std::endl;
#endif /* DEBUG */
  _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();
  _Fl_Overlay_WindowPtr->redraw_overlay ( );
  return octave_value();
}

DEFUN_DLD( Fl_Overlay_Window_show , args , ,
	   " void Fl_Overlay_Window_show (); " )
{
  Fl_Overlay_Window *   _Fl_Overlay_WindowPtr ;
  Cell  matrix;

  if(args.length()!= 2 )
    {
      std::cout<< "eg:  void Fl_Overlay_Window_show (cell {}); " <<std::endl;
      return octave_value(-1);
    }

#ifdef DEBUG
  std::cout<<" Fl_Overlay_Window_show "<<std::endl;
#endif /* DEBUG */

  _Fl_Overlay_WindowPtr =( Fl_Overlay_Window * )args( 0 ).long_value();

  if(args.length()== 1)
    {
      _Fl_Overlay_WindowPtr->show ();
      return octave_value();
    }

  matrix=args(1).cell_value();
  char *list_args[matrix.cols()];

  for(int i=0;i<matrix.cols();i++)
    {
      list_args[i]=strdup(matrix(0,i).string_value().c_str());
      matrix(0,i).print(std::cout,true);
    }

  _Fl_Overlay_WindowPtr->show (matrix.cols(),list_args);
  return octave_value();
}

/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Overlay_Window , args , , " void Fl_Overlay_Window(void); " )
{

  /* Overlay is an abstract virutal class */
#if 0
  if (args.length() == 5 || args.length() == 3)
    {
      char *str=NULL;      
      Fl_Overlay_Window *flb;

      if(args.length()==5)
	{
	  str=strdup(args(4).string_value().c_str());
	  flb=new Fl_Overlay_Window(args(0).int_value(),
				    args(1).int_value(),
				    args(2).int_value(),
				    args(3).int_value(),str);
	}
      else 
	{
	  str=strdup(args(4).string_value().c_str());
	  flb=new Fl_Overlay_Window(args(0).int_value(),
				    args(1).int_value(),
				    str);
	}
      
      return octave_value(new FLTKWrapper(flb,"Fl_Overlay_Window"));
    }
#endif

octave_dynamic_loader::load("Fl_Overlay_Window_flush" ,"");
octave_dynamic_loader::load("Fl_Overlay_Window_hide" ,"");
octave_dynamic_loader::load("Fl_Overlay_Window_resize" ,"");
octave_dynamic_loader::load("Fl_Overlay_Window_can_do_overlay" ,"");
octave_dynamic_loader::load("Fl_Overlay_Window_redraw_overlay" ,"");
octave_dynamic_loader::load("Fl_Overlay_Window_show" ,"");

 std::cout<<"Code generated onSat Jul 30 23:28:33 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_Overlay_WindowLibrary!\n";
 
return octave_value(new FLTKStaticWrapper(0,"Fl_Overlay_Window"));
}
