/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}

DEFUN_DLD( Fl_Pixmap_copy , args , , " virtual Fl_Pixmap_*copy (int W,int H); " )
{
Fl_Pixmap *   _Fl_PixmapPtr ;
int   W ;
int   H ;
 if(args.length()!= 3 || args.length()!=1)
  {
   std::cout<< "eg:  virtual Fl_Pixmap_*copy (int W,int H); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Pixmap_*copy "<<std::endl;
#endif /* DEBUG */
 _Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();

 Fl_Image *ptr=NULL;

 if(args.length()!=1)
   { 
     W = (  int  ) args( 1 ).int_value();
     H = (  int  ) args( 2 ).int_value();
     ptr= _Fl_PixmapPtr->copy ( W,H);
   }
 else
   ptr= _Fl_PixmapPtr->copy ( );

 return octave_value(new FLTKWrapper(ptr,"Fl_Image"));
}


DEFUN_DLD( Fl_Pixmap_color_average , args , , " virtual Fl_Pixmap_color_average (Fl_Color c,float i); " )
{
Fl_Pixmap *   _Fl_PixmapPtr ;
Fl_Color   c ;
float   i ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  virtual Fl_Pixmap_color_average (Fl_Color c,float i); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Pixmap_color_average "<<std::endl;
#endif /* DEBUG */
_Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();
c = (  Fl_Color  ) args( 1 ).long_value();
i = (  float  ) args( 2 ).double_value();
 _Fl_PixmapPtr->color_average ( c,i);
 return octave_value();
}
DEFUN_DLD( Fl_Pixmap_desaturate , args , , " virtual Fl_Pixmap_desaturate ( ); " )
{
Fl_Pixmap *   _Fl_PixmapPtr ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  virtual Fl_Pixmap_desaturate ( ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Pixmap_desaturate "<<std::endl;
#endif /* DEBUG */
 _Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();
 _Fl_PixmapPtr->desaturate ( );
 return octave_value();
}

DEFUN_DLD( Fl_Pixmap_draw , args , , " virtual Fl_Pixmap_draw (int X,int Y,int W,int H,int cx=0,int cy=0); " )
{
Fl_Pixmap *   _Fl_PixmapPtr ;
int   X ;
int   Y ;
int   W =0;
int   H =0;
int   cx=0 ;
int   cy=0 ;
 if(args.length()!= 7 || args.length()==5)
  {
   std::cout<< "eg:  virtual Fl_Pixmap_draw (int X,int Y,int W,int H,int cx=0,int cy=0); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Pixmap_draw "<<std::endl;
#endif /* DEBUG */
_Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();
X = (  int  ) args( 1 ).int_value();
Y = (  int  ) args( 2 ).int_value();

 if(args.length()!= 3 )
  {
    W = (  int  ) args( 3 ).int_value();
    H = (  int  ) args( 4 ).int_value();
    _Fl_PixmapPtr->draw ( X,Y);
  }
 else
   {
     if(args.length()!=5)
       { 
	 cx= (  int  ) args( 5 ).int_value();
	 cy= (  int  ) args( 6 ).int_value();	 
       }
     _Fl_PixmapPtr->draw ( X,Y,W,H,cx,cy);
   }
 return octave_value();
}

DEFUN_DLD( Fl_Pixmap_label , args , ,
	   " virtual Fl_Pixmap_label (Fl_Menu_Item*m ); " )
{
  Fl_Pixmap *   _Fl_PixmapPtr ;
  Fl_Menu_Item*   _Fl_Menu_Itemm0 ;
  Fl_Widget*   _Fl_Widgetw0 ;
  
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  virtual Fl_Pixmap_label (Fl_Menu_Item*m ); " <<std::endl;
      return octave_value(-1);
    }
  
#ifdef DEBUG
  std::cout<<" Fl_Pixmap_label "<<std::endl;
#endif /* DEBUG */

  _Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();
  
  if(FLTKWrapper::ExistObjectType((void *)args(1).long_value(),"Fl_Widget"))
    {
      _Fl_Widgetw0 =( Fl_Widget* )args( 1 ).long_value();
      _Fl_PixmapPtr->label ( _Fl_Widgetw0);
      return octave_value();
    }
  
  _Fl_Menu_Itemm0 =( Fl_Menu_Item* )args( 1 ).long_value();
 _Fl_PixmapPtr->label ( _Fl_Menu_Itemm0);
  return octave_value();
}

DEFUN_DLD( Fl_Pixmap_uncache , args , , " virtual Fl_Pixmap_uncache ( ); " )
{
  Fl_Pixmap *   _Fl_PixmapPtr ;
  if(args.length()!= 1 )
    {
      std::cout<< "eg:  virtual Fl_Pixmap_uncache ( ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
std::cout<<" Fl_Pixmap_uncache "<<std::endl;
#endif /* DEBUG */
_Fl_PixmapPtr =( Fl_Pixmap * )args( 0 ).long_value();
 _Fl_PixmapPtr->uncache ( );
 return octave_value();
}
/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Pixmap , args , , " void Fl_Pixmap(void); " )
{

  /* Make constructor. ATM difficult to make it */
#if 0
  if (args.length() >= 4 && args.length() <= 5)
    {
      char *str=NULL;      
      if(args.length()==5)
	str=strdup(args(4).string_value().c_str());
      Fl_Pixmap *flb=new Fl_Pixmap(args(0).int_value(),
                             args(1).int_value(),
			     args(2).int_value(),
			     args(3).int_value(),str);
      return octave_value(new FLTKWrapper(flb,"Fl_Pixmap"));
    }
#endif

  octave_dynamic_loader::load("Fl_Pixmap_Fl_Pixmap" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_copy" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_color_average" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_desaturate" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_draw" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_label" ,"");
  octave_dynamic_loader::load("Fl_Pixmap_uncache" ,"");

  std::cout<<"Code generated onSun Jul 31 16:29:12 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_PixmapLibrary!"<<std::endl;
  return octave_value(new FLTKStaticWrapper(0,"Fl_Pixmap"));
}
