/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}
DEFUN_DLD( Fl_Tiled_Image_copy , args , , " virtual Fl_Tiled_Image_*copy (int W,int H); " )
{
Fl_Tiled_Image *   _Fl_Tiled_ImagePtr ;
 int   W ;
 int   H ;

 if(args.length()!= 3 || args.length()!= 1)
  {
    std::cout<< "eg:  virtual Fl_Tiled_Image_*copy (int W,int H); " <<std::endl;
    return octave_value(-1);
  }
#ifdef DEBUG
 std::cout<<" Fl_Tiled_Image_*copy "<<std::endl;
#endif /* DEBUG */
 _Fl_Tiled_ImagePtr =( Fl_Tiled_Image * )args( 0 ).long_value();
 Fl_Image *im=NULL;
 if(args.length()==3)
   {
     W = (  int  ) args( 1 ).int_value();
     H = (  int  ) args( 2 ).int_value();
     im=_Fl_Tiled_ImagePtr->copy ( W,H);
   }
 else 
   {
     im=_Fl_Tiled_ImagePtr->copy();
   }
 
 return octave_value(new FLTKWrapper(im,"Fl_Tiled_Image"));
}
DEFUN_DLD( Fl_Tiled_Image_color_average , args , , 
	   " virtual Fl_Tiled_Image_color_average (Fl_Color c,float i); " )
{
  Fl_Tiled_Image *   _Fl_Tiled_ImagePtr ;
  Fl_Color   c ;
  float   i ;
  if(args.length()!= 3 )
    {
      std::cout<< "eg:  virtual Fl_Tiled_Image_color_average (Fl_Color c,float i); " <<std::endl;
      return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Tiled_Image_color_average "<<std::endl;
#endif /* DEBUG */
_Fl_Tiled_ImagePtr =( Fl_Tiled_Image * )args( 0 ).long_value();
c = (  Fl_Color  ) args( 1 ).long_value();
i = (  float  ) args( 2 ).double_value();
 _Fl_Tiled_ImagePtr->color_average ( c,i);
return octave_value();
}

DEFUN_DLD( Fl_Tiled_Image_desaturate , args , , " virtual Fl_Tiled_Image_desaturate ( ); " )
{
Fl_Tiled_Image *   _Fl_Tiled_ImagePtr ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  virtual Fl_Tiled_Image_desaturate ( ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Tiled_Image_desaturate "<<std::endl;
#endif /* DEBUG */
 _Fl_Tiled_ImagePtr =( Fl_Tiled_Image * )args( 0 ).long_value();
 _Fl_Tiled_ImagePtr->desaturate ( );
 return octave_value();
}

DEFUN_DLD( Fl_Tiled_Image_draw , args , , 
	   " virtual Fl_Tiled_Image_draw (int X,int Y,int W,int H,int cx,int cy); " )
{
  Fl_Tiled_Image *   _Fl_Tiled_ImagePtr ;
  int   X =0;
  int   Y =0;
  int   W =0;
  int   H =0;
  int   cx =0;
  int   cy =0;
  if(args.length()!= 7 || args.length()==3)
    {
      std::cout<< "eg:  virtual Fl_Tiled_Image_draw (int X,int Y,int W,int H,int cx,int cy); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Tiled_Image_draw "<<std::endl;
#endif /* DEBUG */
  _Fl_Tiled_ImagePtr =( Fl_Tiled_Image * )args( 0 ).long_value();
  
  if(args.length()==7)
    { 
      X = (  int  ) args( 1 ).int_value();
      Y = (  int  ) args( 2 ).int_value();
      W = (  int  ) args( 3 ).int_value();
      H = (  int  ) args( 4 ).int_value();
     cx = (  int  ) args( 5 ).int_value();
     cy = (  int  ) args( 6 ).int_value();
     _Fl_Tiled_ImagePtr->draw ( X,Y,W,H,cx,cy);
    }
  else
    _Fl_Tiled_ImagePtr->draw ( X,Y);
  
  return octave_value();
}

DEFUN_DLD( Fl_Tiled_Image_image , args , , " Fl_Image * Fl_Tiled_Image_image ( ); " )
{
Fl_Tiled_Image *   _Fl_Tiled_ImagePtr ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  Fl_Image * Fl_Tiled_Image_image ( ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Tiled_Image_image "<<std::endl;
#endif /* DEBUG */
_Fl_Tiled_ImagePtr =( Fl_Tiled_Image * )args( 0 ).long_value();
Fl_Image *im= _Fl_Tiled_ImagePtr->image ( );
return octave_value(new FLTKWrapper(im,"Fl_Image"));
}
/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Tiled_Image , args , , " void Fl_Tiled_Image(void); " )
{
  if (args.length() >= 1 && args.length() <= 3)
    {
      Fl_Image *i=NULL;
      int W=0;
      int H=0;
      /*Fl_Tiled_Image(Fl_Image *i, int W = 0, int H = 0);*/
      i=(Fl_Image *)args(0).long_value();
      if(args.length()>=2)
	W=args(1).long_value();      
      if(args.length()>=3)
	W=args(2).long_value();
      
      Fl_Tiled_Image *flb=new Fl_Tiled_Image(i,W,H);
      return octave_value(new FLTKWrapper(flb,"Fl_Tiled_Image"));
    }
  
  octave_dynamic_loader::load("Fl_Tiled_Image_copy" ,"");
  octave_dynamic_loader::load("Fl_Tiled_Image_color_average" ,"");
  octave_dynamic_loader::load("Fl_Tiled_Image_desaturate" ,"");
  octave_dynamic_loader::load("Fl_Tiled_Image_draw" ,"");
  octave_dynamic_loader::load("Fl_Tiled_Image_image" ,"");
  std::cout<<"Code generated onMon Aug  1 01:17:33 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_Tiled_ImageLibrary!"<<std::endl;
  return octave_value(new FLTKStaticWrapper(0,"Fl_Tiled_Image"));
}
