/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}

DEFUN_DLD( Fl_Valuator_bounds , args , , " void Fl_Valuator_bounds (double a,double b); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
double   a ;
double   b ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  void Fl_Valuator_bounds (double a,double b); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Valuator_bounds "<<std::endl;
#endif /* DEBUG */
_Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
a = (  double  ) args( 1 ).double_value();
b = (  double  ) args( 2 ).double_value();
_Fl_ValuatorPtr->bounds ( a,b);
return octave_value();
}

DEFUN_DLD( Fl_Valuator_minimum , args , , " void Fl_Valuator_minimum (double a); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
double   a ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  void Fl_Valuator_minimum (double a); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Valuator_minimum "<<std::endl;
#endif /* DEBUG */
_Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
if(args.length()!= 1)
{
  a = (  double  ) args( 1 ).double_value();
  _Fl_ValuatorPtr->minimum ( a);
  return octave_value();
}
long int ret_type=(long int) _Fl_ValuatorPtr->minimum ( );
return octave_value(ret_type);
}


DEFUN_DLD( Fl_Valuator_maximum , args , , " void Fl_Valuator_maximum (double a); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
double   a ;
 if(args.length()!= 2 || args.length()!=1)
  {
   std::cout<< "eg:  void Fl_Valuator_maximum (double a); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Valuator_maximum "<<std::endl;
#endif /* DEBUG */

_Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();

if(args.length()!= 1 )
  {
    a = (  double  ) args( 1 ).double_value();
     _Fl_ValuatorPtr->maximum ( a);
    return octave_value();
  }
 long int ret_type=(long int) _Fl_ValuatorPtr->maximum ( );
 return octave_value(ret_type);
}
DEFUN_DLD( Fl_Valuator_range , args , , " void Fl_Valuator_range (double a,double b); " )
{
  Fl_Valuator *   _Fl_ValuatorPtr ;
 double   a ;
 double   b ;
 if(args.length()!= 3)
  {
   std::cout<< "eg:  void Fl_Valuator_range (double a,double b); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Valuator_range "<<std::endl;
#endif /* DEBUG */
_Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
a = (  double  ) args( 1 ).double_value();
b = (  double  ) args( 2 ).double_value();
_Fl_ValuatorPtr->range ( a,b);
return octave_value();
}

DEFUN_DLD( Fl_Valuator_step , args , , " void Fl_Valuator_step (double a,int b); " )
{
  Fl_Valuator *   _Fl_ValuatorPtr ;
  double   a ;
  int   b ;
  if(args.length()>=1 && args.length() <= 3 )
    {
      std::cout<< "eg:  void Fl_Valuator_step (double a,int b); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Valuator_step "<<std::endl;
#endif /* DEBUG */
  
  _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
  switch(args.length())
    {
    case 3:
      a = (  double  ) args( 1 ).double_value();
      b = (  int  ) args( 2 ).int_value();
      _Fl_ValuatorPtr->step ( a,b);
      return octave_value();
      break;
    case 2:
      if(args(1).long_value() == (long int)args(1).double_value())
	{
	  a = (  int  ) args( 1 ).int_value(); 
	  _Fl_ValuatorPtr->step ( a);
	}
      else
	{
	  double d = (  int  ) args( 1 ).double_value(); 
	  _Fl_ValuatorPtr->step ( d);
	}
      return octave_value();
      break;
    case 1:
      double d=_Fl_ValuatorPtr->step ( );
      return octave_value(d);
      break;
    }
  // Unreachable code!
  return octave_value();
}

DEFUN_DLD( Fl_Valuator_precision , args , , " void Fl_Valuator_precision (int ); " )
{
  Fl_Valuator *   _Fl_ValuatorPtr ;
  int   _int0 ;
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  void Fl_Valuator_precision (int ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Valuator_precision "<<std::endl;
#endif /* DEBUG */
  _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
  _int0 = (  int  ) args( 1 ).int_value();
  _Fl_ValuatorPtr->precision ( _int0);
  return octave_value();
}

DEFUN_DLD( Fl_Valuator_value , args , , " int Fl_Valuator_value (double ); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
double   _double0 ;
 if(args.length()!= 2 || args.length()!= 1)
  {
   std::cout<< "eg:  int Fl_Valuator_value (double ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
 std::cout<<" Fl_Valuator_value "<<std::endl;
#endif /* DEBUG */
 _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
 if( args.length()!= 1)
   {
     _double0 = (  double  ) args( 1 ).double_value();
     long int ret_type=(long int) _Fl_ValuatorPtr->value ( _double0);
     return octave_value(ret_type);
   }
 
 long int ret_type=(long int) _Fl_ValuatorPtr->value ( );
 return octave_value(ret_type);
}

DEFUN_DLD( Fl_Valuator_format , args , , 
	   " virtual Fl_Valuator_format (char* ); " )
{
  Fl_Valuator *   _Fl_ValuatorPtr ;
  char*   _char0 ;
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  virtual Fl_Valuator_format (char* ); " <<std::endl;
      return octave_value(-1);
    }

#ifdef DEBUG
  std::cout<<" Fl_Valuator_format "<<std::endl;
#endif /* DEBUG */

  _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
  _char0 =( char* )args( 1 ).long_value();
  long int ret_type=(long int) _Fl_ValuatorPtr->format ( _char0);

  return octave_value(ret_type);
}

DEFUN_DLD( Fl_Valuator_round , args , ,
	   " double Fl_Valuator_round (double ); " )
{
  Fl_Valuator *   _Fl_ValuatorPtr ;
  double   _double0 ;
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  double Fl_Valuator_round (double ); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_Valuator_round "<<std::endl;
#endif /* DEBUG */
  _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
  _double0 = (  double  ) args( 1 ).double_value();
  long int ret_type=(long int) _Fl_ValuatorPtr->round ( _double0);
  return octave_value(ret_type);
}

DEFUN_DLD( Fl_Valuator_clamp , args , , " double Fl_Valuator_clamp (double ); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
double   _double0 ;
 if(args.length()!= 2 )
  {
   std::cout<< "eg:  double Fl_Valuator_clamp (double ); " <<std::endl;
   return octave_value(-1);
  }

#ifdef DEBUG
 std::cout<<" Fl_Valuator_clamp "<<std::endl;
#endif /* DEBUG */

 _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
 _double0 = (  double  ) args( 1 ).double_value();
 long int ret_type=(long int) _Fl_ValuatorPtr->clamp ( _double0);
 return octave_value(ret_type);
}

DEFUN_DLD( Fl_Valuator_increment , args , , " double Fl_Valuator_increment (double ,int ); " )
{
Fl_Valuator *   _Fl_ValuatorPtr ;
 double   _double0 ;
 int   _int1 ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  double Fl_Valuator_increment (double ,int ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_Valuator_increment "<<std::endl;
#endif /* DEBUG */
 _Fl_ValuatorPtr =( Fl_Valuator * )args( 0 ).long_value();
 _double0 = (  double  ) args( 1 ).double_value();
 _int1 = (  int  ) args( 2 ).int_value();
 long int ret_type=(long int) _Fl_ValuatorPtr->increment ( _double0,_int1);
 return octave_value(ret_type);
}

/* Overridden functions */
/* Linked Separately */
DEFUN_DLD( Fl_Valuator , args , , " void Fl_Valuator(void); " )
{
  /*
    Constructor is at  a protected Level!
  */

  /* public methods */
  octave_dynamic_loader::load("Fl_Valuator_bounds" ,"");
  octave_dynamic_loader::load("Fl_Valuator_minimum" ,"");
  octave_dynamic_loader::load("Fl_Valuator_maximum" ,"");
  octave_dynamic_loader::load("Fl_Valuator_range" ,"");
  octave_dynamic_loader::load("Fl_Valuator_step" ,"");
  octave_dynamic_loader::load("Fl_Valuator_precision" ,"");
  octave_dynamic_loader::load("Fl_Valuator_value" ,"");
  octave_dynamic_loader::load("Fl_Valuator_format" ,"");
  octave_dynamic_loader::load("Fl_Valuator_round" ,"");
  octave_dynamic_loader::load("Fl_Valuator_clamp" ,"");
  octave_dynamic_loader::load("Fl_Valuator_increment" ,"");
  std::cout<<"Code generated onSat Jul 30 23:28:35 2005\n Compiled on " __DATE__  __TIME__ "\n loadedFl_ValuatorLibrary!\n";
  
  return octave_value();
}
