/* Standard headers std.header */
/* Standard headers std.header */
/*
 * This program is free software.
 * This code is a part of Octave-FLTK
 * Code may be used or distributed under GPL.
 * (C) July 2005 Muthiah Annamalai, Octave-FLTK
 */

/*
 *    Octave-FLTK
 *       The GUI bridge from octave.
 *    Copyright (C) July 2005 Muthiah Annamalai, Octave-FLTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>


/* FLTK specific includes */
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include "FLTKWrapper.h"

using namespace std;

extern "C"
{
#include<stdio.h>
#include<string.h>
}
DEFUN_DLD( Fl_beep , args , , " void Fl_beep (int type = FL_BEEP_DEFAULT); " )
{
int   type = FL_BEEP_DEFAULT ;
 if(args.length()!= 1 && args.length()==0 )
  {
   std::cout<< "eg:  void Fl_beep (int type = FL_BEEP_DEFAULT); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_beep "<<std::endl;
#endif /* DEBUG */

 if(args.length()==0) 
   type = FL_BEEP_DEFAULT;
 else
   type = (  int  ) args( 0 ).int_value();
 fl_beep ( type);
return octave_value();
}

DEFUN_DLD( Fl_message , args , , " void Fl_message (char * ,char * varargs_value); " )
{
  char *   _char0 ;
  char *   varargs_value ;
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  void Fl_message (char * ,char * varargs_value); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_message "<<std::endl;
#endif /* DEBUG */
  _char0 = new  char   [args( 0 ).string_value ().length()];
  strcpy( (char *) _char0 ,args( 0 ).string_value().c_str());
  varargs_value = new  char   [args( 1).string_value ().length()];
  strcpy( (char *) varargs_value ,args( 1 ).string_value().c_str());
  fl_message ( _char0,varargs_value);
  delete varargs_value;
  delete _char0;
  return octave_value();
}

DEFUN_DLD( Fl_alert , args , , " void Fl_alert (char * ,char * varargs_value); " )
{
  char *   _char0 ;
  char *   varargs_value ;
  if(args.length()!= 3 )
    {
      std::cout<< "eg:  void Fl_alert (char * ,char * varargs_value); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_alert "<<std::endl;
#endif /* DEBUG */
  _char0 = new  char   [args( 0 ).string_value ().length()];
  strcpy( (char *) _char0 ,args( 0 ).string_value().c_str());
  varargs_value = new  char   [args( 1 ).string_value ().length()];
  strcpy( (char *) varargs_value ,args( 1 ).string_value().c_str());
  fl_alert ( _char0,varargs_value);
  return octave_value();
}

DEFUN_DLD( Fl_ask , args , , " int Fl_ask (char * ,char * varargs_value); " )
{

  char *   _char0 ;
  char *   varargs_value ;
  if(args.length()!= 2 )
    {
      std::cout<< "eg:  int Fl_ask (char * ,char * varargs_value); " <<std::endl;
      return octave_value(-1);
    }
#ifdef DEBUG
  std::cout<<" Fl_ask "<<std::endl;
#endif /* DEBUG */
  _char0 = new  char   [args( 0 ).string_value ().length()];
  strcpy( (char *) _char0 ,args( 0 ).string_value().c_str());
  varargs_value = new  char   [args( 1 ).string_value ().length()];
  strcpy( (char *) varargs_value ,args( 1 ).string_value().c_str());
  long int ret_type=(long int) fl_ask ( _char0,varargs_value);
  return octave_value(ret_type);
}

DEFUN_DLD( Fl_choice , args , , " int Fl_choice (char * q,char * b0,char * b1,char * b2,char * varargs_value); " )
{
char *   q ;
char *   b0 ;
char *   b1 ;
char *   b2 ;
char *   varargs_value ;
 if(args.length()!= 5 )
  {
   std::cout<< "eg:  int Fl_choice (char * q,char * b0,char * b1,char * b2,char * varargs_value); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_choice "<<std::endl;
#endif /* DEBUG */
q = new  char   [args( 0 ).string_value ().length()];
strcpy( (char *) q ,args( 0 ).string_value().c_str());
b0 = new  char   [args( 1 ).string_value ().length()];
strcpy( (char *) b0 ,args( 1 ).string_value().c_str());
b1 = new  char   [args( 2 ).string_value ().length()];
strcpy( (char *) b1 ,args( 2 ).string_value().c_str());
b2 = new  char   [args( 3 ).string_value ().length()];
strcpy( (char *) b2 ,args( 3 ).string_value().c_str());
varargs_value = new  char   [args( 4 ).string_value ().length()];
strcpy( (char *) varargs_value ,args( 4 ).string_value().c_str());
long int ret_type=(long int) fl_choice ( q,b0,b1,b2,varargs_value);
return octave_value(ret_type);
}
DEFUN_DLD( Fl_input , args , , " char * Fl_input (char * label,char * deflt = 0,char * varargs_value); " )
{
char *   label ;
char *   deflt;
char *   varargs_value ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  char * Fl_input (char * label,char * deflt = 0,char * varargs_value); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_input "<<std::endl;
#endif /* DEBUG */
label = new  char   [args( 0 ).string_value ().length()];
strcpy( (char *) label ,args( 0 ).string_value().c_str());
deflt = new  char   [args( 1 ).string_value ().length()];
strcpy( (char *) deflt ,args( 1 ).string_value().c_str());
varargs_value = new  char   [args( 2 ).string_value ().length()];
strcpy( (char *) varargs_value ,args( 2 ).string_value().c_str());
string  ret_type(  (const char *) fl_input ( label,deflt ,varargs_value));
return octave_value(ret_type);
}
DEFUN_DLD( Fl_password , args , , " char * Fl_password (char * label,char * deflt = 0,char * varargs_value); " )
{
char *   label ;
char *   deflt  ;
char *   varargs_value ;
 if(args.length()!= 3 )
  {
   std::cout<< "eg:  char * Fl_password (char * label,char * deflt = 0,char * varargs_value); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_password "<<std::endl;
#endif /* DEBUG */
label = new  char   [args( 0 ).string_value ().length()];
strcpy( (char *) label ,args( 0 ).string_value().c_str());
deflt = new  char   [args( 1 ).string_value ().length()];
strcpy( (char *) deflt  ,args( 1 ).string_value().c_str());
varargs_value = new  char   [args( 2 ).string_value ().length()];
strcpy( (char *) varargs_value ,args( 3 ).string_value().c_str());
string  ret_type(  (const char *) fl_password ( label,deflt ,varargs_value));
return octave_value(ret_type);
}
DEFUN_DLD( Fl_message_icon , args , , " Fl_Widget * Fl_message_icon ( ); " )
{
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  Fl_Widget * Fl_message_icon ( ); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" Fl_message_icon "<<std::endl;
#endif /* DEBUG */
/* Nothing! */
long int ret_type= (long int) fl_message_icon ( );
return octave_value(ret_type);
}
/* Overridden functions */
/* Linked Separately */
DEFUN_DLD(fl_ask, args , , " void -generate_static_code(void); " )
{

  octave_dynamic_loader::load("Fl_beep" ,"");
  octave_dynamic_loader::load("Fl_message" ,"");
  octave_dynamic_loader::load("Fl_alert" ,"");
  octave_dynamic_loader::load("Fl_ask" ,"");
  octave_dynamic_loader::load("Fl_choice" ,"");
  octave_dynamic_loader::load("Fl_input" ,"");
  octave_dynamic_loader::load("Fl_password" ,"");
  octave_dynamic_loader::load("Fl_message_icon" ,"");

  std::cout<<"Code generated onMon Aug  1 20:20:24 2005\n Compiled on " __DATE__  __TIME__ "\n loaded-generate_static_codeLibrary!"<<std::endl;
  return octave_value(new FLTKStaticWrapper(0,"Fl"));
}
